@@ -198,7 +198,7 @@ void binseg_var_norm(y2,n,pen,Q,cptsout,likeout,op_cps)
 
 
 double mll_var(double x, int n){
-	if(x<0){x=0.00000000001;}
+	if(x<=0){x=0.00000000001;}
 	return(n*(log(2*M_PI)+log(x/n)+1)); /* M_PI is in Rmath.h  */
 }
 
@@ -375,6 +375,7 @@ void binseg_meanvar_norm(y2,y,n,pen,Q,cptsout,likeout,op_cps)
 
 double mll_meanvar(double x, double x2, int n){
 	double sigsq=(x2-((x*x)/n))/n;
+	if(sigsq<=0){sigsq=0.00000000001;}
 	return(n*(log(2*M_PI)+log(sigsq)+1)); /* M_PI is in Rmath.h  */
 }
 
@@ -905,6 +906,182 @@ double mll_meanvar_gamma(double x, int n,double shape){
 
 
 
+void PELT_meanvar_poisson(y,n,pen,cptsout,error)
+	double *y;		/*Summary statistics for the time series */
+	int *n;			/* Length of the time series */
+  double *pen;  /* Penalty used to decide if a changepoint is significant */
+  int *cptsout;    /* Vector of identified changepoint locations */
+  int *error;   /* 0 by default, nonzero indicates error in code */
+  {
+	// R code does know.mean and fills mu if necessary
+
+	//int lastchangecpts[*n][2]; /* stores last changepoint locations   */
+  int *lastchangecpts;
+  lastchangecpts = (int *)calloc(*n*2,sizeof(int));
+  if (lastchangecpts==NULL)   {
+    *error = 1;
+    goto err1;
+  }
+
+  //double lastchangelike[*n]; /* stores likelihood up to that time using optimal changepoint locations up to that time */
+  double *lastchangelike;
+  lastchangelike = (double *)calloc(*n,sizeof(double));
+  if (lastchangelike==NULL)   {
+    *error = 2;
+    goto err2;
+  }
+
+  //int checklist[*n];
+  int *checklist;
+  checklist = (int *)calloc(*n,sizeof(int));
+  if (checklist==NULL)   {
+    *error = 3;
+    goto err3;
+  }
+
+  int nchecklist;
+	double minout;
+
+  //double tmplike[*n];
+  double *tmplike;
+  tmplike = (double *)calloc(*n,sizeof(double));
+  if (tmplike==NULL)   {
+    *error = 4;
+    goto err4;
+  }
+
+	//int tmpt[*n];
+  int *tmpt;
+  tmpt = (int *)calloc(*n,sizeof(int));
+  if (tmpt==NULL)   {
+    *error = 5;
+    goto err5;
+  }
+
+	int tstar,i,whichout,nchecktmp;
+
+	double mll_meanvar_poisson();
+	void min_which();
+	
+	lastchangelike[0]=mll_meanvar_poisson(*(y+1),1);
+	lastchangecpts[0]=0; lastchangecpts[*n+0]=1;
+	lastchangelike[1]=mll_meanvar_poisson(*(y+2),2);
+	lastchangecpts[1]=0; lastchangecpts[*n+1]=2;
+	lastchangelike[2]=mll_meanvar_poisson(*(y+3),3);
+	lastchangecpts[2]=0; lastchangecpts[*n+2]=3;
+
+	nchecklist=1;
+	checklist[0]=2;
+	for(tstar=4;tstar<(*n+1);tstar++){
+    R_CheckUserInterrupt(); /* checks if user has interrupted the R session and quits if true */
+
+    for(i=0;i<nchecklist;i++){
+			tmplike[i]=lastchangelike[checklist[i]-1] + mll_meanvar_poisson(*(y+tstar)-*(y+checklist[i]),tstar-checklist[i])+*pen;
+		}
+		tmplike[nchecklist]=mll_meanvar_poisson(*(y+tstar),tstar);
+		min_which(tmplike,nchecklist+1,&minout,&whichout); /*updates minout and whichout with min and which element */
+		lastchangelike[tstar-1]=minout;
+		if(whichout==nchecklist){		lastchangecpts[tstar-1]=0; lastchangecpts[*n+tstar-1]=tstar; } /* Null */
+		else{			lastchangecpts[tstar-1]=checklist[whichout]; lastchangecpts[*n+tstar-1]=tstar; } /* Alt */
+		
+		/* Update checklist for next iteration, first element is next tau */
+		nchecktmp=0;
+		for(i=0;i<nchecklist;i++){
+			if(tmplike[i]<= (lastchangelike[tstar-1]+*pen)){
+				*(checklist+nchecktmp)=checklist[i];
+				nchecktmp+=1;
+			}
+		}
+		*(checklist+nchecktmp)=tstar-1;  // atleast 2 obs per seg
+		nchecktmp+=1;
+		nchecklist=nchecktmp;
+	} // end taustar
+	
+	// put final set of changepoints together
+	int ncpts=0;
+	int last=*n;
+	while(last!=0){
+		*(cptsout+ncpts)=lastchangecpts[*n+last-1];
+		last=lastchangecpts[last-1];
+		ncpts+=1;
+	}
+  free(tmpt);
+err5:  free(tmplike);
+err4:  free(checklist);
+err3:  free(lastchangelike);
+err2:  free(lastchangecpts);
+err1:  return;
+}
+
+
+void binseg_meanvar_poisson(y,n,pen,Q,cptsout,likeout,op_cps)
+	double *y;    /* Summary statistic for the time series */
+	int *n;			/* Length of the time series */
+  double *pen;  /* Penalty used to decide if a changepoint is significant */
+	int *Q;			/* Max number of changepoints */
+  int *cptsout;    /* Q length vector of identified changepoint locations */
+	double *likeout;		/* Q length vector of likelihood ratio values for changepoints in cptsout */
+	int *op_cps;		/* Optimal number of changepoint for pen supplied */
+  {
+	// must -0.5*mll_var to get same as R code
+
+	double oldmax=1000,null,lambda[*n],maxout;
+	int q,p,i,j,whichout,st,end;
+	int tau[*Q+2]; // max ncpts is Q, +2 is for 0 and n
+	tau[0]=0;
+	tau[1]= *n;
+
+	double mll_meanvar_poisson();
+	void max_which();
+	void order_vec();
+
+  for(q=0;q<*Q;q++){
+    R_CheckUserInterrupt(); /* checks if user has interrupted the R session and quits if true */
+
+    for(p=0;p<*n;p++){lambda[p]=0;}
+    i=1;
+    st=tau[0]+1;
+		end=tau[1];
+    null= (-0.5) * mll_meanvar_poisson(*(y+end)-*(y+st-1),end-st+1);
+    for(j=2;j<(*n-2);j++){
+      if(j==end){
+        st=end+1;
+				i=i+1;
+				end=tau[i];
+        null= (-0.5) * mll_meanvar_poisson(*(y+end)-*(y+st-1),end-st+1);
+      }
+			else{
+				if(((j-st)>1)&&((end-j)>1)){
+	        lambda[j]= ((-0.5) * mll_meanvar_poisson(*(y+j)-*(y+st-1),j-st+1)) + ((-0.5) * mll_meanvar_poisson(*(y+end)-*(y+j),end-j)) - null;
+				}
+      }
+    }
+    max_which(lambda,*n,&maxout,&whichout);
+
+    *(cptsout+q)=whichout;
+		*(likeout+q)= (oldmax<=maxout) ? oldmax : maxout ;
+    oldmax= *(likeout+q);
+		tau[q+2]=whichout;
+    order_vec(tau,q+3);
+  }
+
+	int stop=0;
+	*op_cps=0;
+	while((stop==0)&&(*op_cps < *Q)){
+		if((2* *(likeout+*op_cps)) >= *pen){ (*op_cps)++; }
+		else{ stop=1; }
+	}
+}
+
+
+double mll_meanvar_poisson(double x, int n){
+	if(x==0){return(INFINITY);}
+	else{return(2*x*(log(n)-log(x)));}
+}
+
+
+
+
 
 void min_which(double *array,int n,double *minout,int *whichout){
 	/* Function to find minimum of an array with n elements that is put in min */
